/*
	This file is part of Fennix C Library.

	Fennix C Library is free software: you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation, either version 3 of
	the License, or (at your option) any later version.

	Fennix C Library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Fennix C Library. If not, see <https://www.gnu.org/licenses/>.
*/

#include <bits/libc.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <errno.h>

export int creat(const char *path, mode_t mode) { return open(path, O_WRONLY | O_CREAT | O_TRUNC, mode); }

export int fcntl(int fildes, int cmd, ...)
{
	va_list args;
	va_start(args, cmd);
	void *arg = va_arg(args, void *);
	va_end(args);
	return __check_errno(sysdep(FileControl)(fildes, cmd, arg), -1);
}

export int open(const char *path, int oflag, ...)
{
	mode_t mode = 0;
	if (oflag & O_CREAT)
	{
		va_list args;
		va_start(args, oflag);
		mode = va_arg(args, mode_t);
		va_end(args);
	}
	return __check_errno(sysdep(Open)(path, oflag, mode), -1);
}

export int openat(int fd, const char *path, int oflag, ...);
export int posix_fadvise(int fd, off_t offset, off_t len, int advice);
export int posix_fallocate(int fd, off_t offset, off_t len);
